##############################################################################
#
# Copyright (c) 2002 Nexedi SARL and Contributors. All Rights Reserved.
#                    Jean-Paul Smets-Solane <jp@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFMailIn.MailMessage import MailMessage
from Products.ERP5Type import Permissions, PropertySheet, Constraint, Interface

from Event import Event
import smtplib

class MailMessage(MailMessage, Event):
    """
      MailMessage subclasses Event objects to implement Email Events.
    """

    meta_type = 'ERP5 Mail Message'
    portal_type = 'Mail Message'
    add_permission = Permissions.AddERP5Content
    isPortalContent = 1
    isRADContent = 1

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(Permissions.View)

    # Declarative properties
    property_sheets = ( PropertySheet.Base
                      , PropertySheet.CategoryCore
                      , PropertySheet.DublinCore
                      , PropertySheet.Task
                      )

    # Factory Type Information
    factory_type_information = \
      {    'id'             : portal_type
         , 'meta_type'      : meta_type
         , 'description'    : """\
An Event object holds the information about
an event."""
         , 'icon'           : 'event_icon.gif'
         , 'product'        : 'ERP5'
         , 'factory'        : 'addMailMessage'
         , 'immediate_view' : 'mail_message_view'
         , 'actions'        :
        ( { 'id'            : 'view'
          , 'name'          : 'View'
          , 'category'      : 'object_view'
          , 'action'        : 'mail_message_view'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'print'
          , 'name'          : 'Print'
          , 'category'      : 'object_print'
          , 'action'        : 'mail_message_print'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'metadata'
          , 'name'          : 'Metadata'
          , 'category'      : 'object_view'
          , 'action'        : 'metadata_edit'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'translate'
          , 'name'          : 'Translate'
          , 'category'      : 'object_action'
          , 'action'        : 'translation_template_view'
          , 'permissions'   : (
              Permissions.TranslateContent, )
          }
        )
      }

    security.declareProtected(Permissions.ModifyPortalContent, 'edit')
    def edit(self,
                 title = '',
                 subject = '',
                 date = '',
                 to = '',
                 sender = '',
                 replyto = '',
                 body = '',
                 headers = '',
                 otherInfo = '',
                 attachments = {},
                 **kw
                 ):
        self.setTitle(title)
        self.setSubject(subject)
        self.date = date
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers
        self.otherInfo = otherInfo
        self.attachments = attachments
        #Event.edit(self,id,**kw)

    def getSubject(self):
       """
       get the Subject
       """
       # We have to convert the Subject wich is list
       # as a string
       result = ''
       if type(self.subject) is type(()):
         for letter in self.subject:
           result += letter
       return result

    def getDate(self):
       """
       get the Date
       """
       return self.date

    def getTo(self):
       """
       get the To
       """
       return self.to

    def getSender(self):
       """
       get the Sender
       """
       return self.sender

    def getReplyTo(self):
       """
       get the Reply To
       """
       return self.replyto

    def getBody(self):
       """
       get the Body
       """
       return self.body

    def getHeader(self):
       """
       get the Header
       """
       return self.headers

    def getOtherInfo(self):
       """
       get Other Info
       """
       return self.otherInfo

    def getAttachment(self):
       """
       get Attachment
       """
       return self.attachments

    def setDate(self, date):
       """
       set the Date
       """
       self.date = date

    def setTo(self, to):
       """
       set the To
       """
       self.to = to

    def setSender(self, sender):
       """
       set the Sender
       """
       self.sender = sender

    def setReplyTo(self, reply_to):
       """
       set the reply_to
       """
       self.replyto = reply_to

    def setBody(self, body):
       """
       set the Body
       """
       self.body = body

    def setHeader(self, header):
       """
       set the Header
       """
       self.headers = header

    def setOtherInfo(self, other_info):
       """
       set Other Info
       """
       self.otherInfo = other_info

    def setAttachment(self, attachment):
       """
       set the Attachement
       """
       self.attachments = attachement

    def SendMail(from_addr=None, smtp_server=None, to_addr=None, msg=None, subject=None):
        """
        Send An Email
        """
        if smtp_server == None:
          smtp_server = 'localhost'
        # We assume by default that we are replying to the sender
        if from_addr == None:
          from_addr = self.getTo()
        if to_addr == None:
          to_addr = self.getSender()
        if msg is not None and subject is not None:
          server = smtplib.SMTP(smtp_server)
          server.sendmail()



