##############################################################################
#
# Copyright (c) 2002 Nexedi SARL and Contributors. All Rights Reserved.
#                    Jean-Paul Smets-Solane <jp@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName

from Products.ERP5Type import Permissions, PropertySheet, Constraint, Interface
from Products.ERP5.Document.Rule import Rule

from zLOG import LOG

class TransformationRule(Rule):
    """
      Order Rule object make sure an Order in the similation
      is consistent with the real order
    """

    # CMF Type Definition
    meta_type = 'ERP5 Transformation Rule'
    portal_type = 'Transformation Rule'
    add_permission = Permissions.AddERP5Content
    isPortalContent = 1
    isRADContent = 1

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(Permissions.View)

    # Default Properties
    property_sheets = ( PropertySheet.Base
                      , PropertySheet.XMLObject
                      , PropertySheet.CategoryCore
                      , PropertySheet.DublinCore
                      )

    # CMF Factory Type Information
    factory_type_information = \
      {    'id'             : portal_type
         , 'meta_type'      : meta_type
         , 'description'    : """\
An ERP5 Rule..."""
         , 'icon'           : 'rule_icon.gif'
         , 'product'        : 'ERP5'
         , 'factory'        : 'addTransformationRule'
         , 'immediate_view' : 'rule_view'
         , 'allow_discussion'     : 1
         , 'allowed_content_types': ()
         , 'filter_content_types' : 1
         , 'global_allow'   : 1
         , 'actions'        :
        ( { 'id'            : 'view'
          , 'name'          : 'View'
          , 'category'      : 'object_view'
          , 'action'        : 'rule_view'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'list'
          , 'name'          : 'Object Contents'
          , 'category'      : 'object_action'
          , 'action'        : 'folder_contents'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'print'
          , 'name'          : 'Print'
          , 'category'      : 'object_print'
          , 'action'        : 'rule_print'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'metadata'
          , 'name'          : 'Metadata'
          , 'category'      : 'object_view'
          , 'action'        : 'metadata_edit'
          , 'permissions'   : (
              Permissions.View, )
          }
        , { 'id'            : 'translate'
          , 'name'          : 'Translate'
          , 'category'      : 'object_action'
          , 'action'        : 'translation_template_view'
          , 'permissions'   : (
              Permissions.TranslateContent, )
          }
        )
      }

    security.declareProtected(Permissions.ModifyPortalContent, 'constructNewAppliedRule')
    def constructNewAppliedRule(self, context, id=None):
      """
        Creates a new applied rule which points to self
      """
      my_applied_rule = Rule.constructNewAppliedRule(self, context, id=id)
      resource = context.getDefaultResourceValue()
      # Find my related transformation
      transformation = resource.getDefaultResourceRelatedValue(portal_type = ('Transformation',))
      if transformation is not None:
        LOG('Transformation Causality', 0, str(transformation.getId()))
        my_applied_rule.setCausalityValue(transformation )
      return my_applied_rule

    security.declareProtected(Permissions.AccessContentsInformation, 'test')
    def test(self, movement):
      """
        Tests if the rule (still) applies
      """
      # Test if we must transform
      # The test should actually be based on nodes, paths
      # and capacities, which is not possible now
      # so we just test if it is a "model" !

      return 0 # for now

      resource = movement.getDefaultResourceValue()
      if resource is None:
        return 0
      module = resource.aq_parent

      if module.id == 'modele':
        # This is the modele !
        # We must also test the nodes
        # if the source is a production node
        return 1
      return 0

    # Simulation workflow
    def reset(self, applied_rule):
      """
        DO WE NEED IT ?

        -> this does either a diverge or a reset depending
        on the position in the tree

        if it is in root position, it is a solve
        if it is in non root position, it is a diverse
      """

    security.declareProtected(Permissions.ModifyPortalContent, 'expand')
    def expand(self, applied_rule):
      """
        Expands the current movement downward.

        -> new status -> expanded

        An applied rule can be expanded only if its parent movement
        is expanded.
      """
      delivery_line_type = 'Simulation Movement'

      # Get the order where we come from
      my_transformation = applied_rule.getDefaultCausalityValue()

      # No transformation defined
      if my_transformation is None:
        # Things stop here
        applied_rule.diverge()
        return

      # Generate production and consumption lines
      my_context_movement = applied_rule.getParent()
      my_quantity = my_context_movement.getQuantity()
      LOG('Transformation', 0, str(my_transformation))
      amount_list , grand_total_base_price, grand_total_source_base_price,\
      grand_total_duration, grand_total_duration_france, grand_total_variated_base_price,\
      grand_total_variated_source_base_price = my_transformation.getAggregatedAmountList(
                                                context = my_context_movement)
                                                # Coramy Specific
      for amount_line in amount_list:
        new_id = applied_rule.generateNewId()
        my_context_movement.portal_types.constructContent(
              type_name = delivery_line_type,
              container = applied_rule,
              id = new_id,
              quantity = amount_line['quantity'] * my_quantity,
              target_quantity = amount_line['quantity'] * my_quantity,
              efficiency = amount_line['efficiency'],
              target_efficiency = amount_line['efficiency'],
              resource_value = amount_line['resource']
          )
        applied_rule[new_id]._setVariationCategoryList(
                amount_line['variation_category_list'])
        # We must set source and destination in this movement
        # as well as source_section and destincation_section
        #
        # source: None destination: the transformation node
        # source_section: the transformation node
        # destination_section: the source_section of the main resource
        # TBD
        applied_rule[new_id]._edit(
              source = None,
              destination = None
              )
        # And define total time
        # By using transformation total time, mult. qty, divided capacity
        # (and mult. by some security factor)
        # This will eventually lead to some kind of conflit
        # which will have to be solved. Solutions include
        #   - flatten (all things in parallel during long time)
        #   - serial (all things one after the other)
        applied_rule[new_id]._edit(
              start_date = None,
              stop_date = None
              )

      # Remove each movement not in the transformation
      # XXX TBD

      # Pass to base class
      Rule.expand(self, applied_rule)

    security.declareProtected(Permissions.ModifyPortalContent, 'solve')
    def solve(self, applied_rule, solution_list):
      """
        Solve inconsitency according to a certain number of solutions
        templates. This updates the

        -> new status -> solved

        This applies a solution to an applied rule. Once
        the solution is applied, the parent movement is checked.
        If it does not diverge, the rule is reexpanded. If not,
        diverge is called on the parent movement.
      """

    security.declareProtected(Permissions.ModifyPortalContent, 'diverge')
    def diverge(self, applied_rule):
      """
        -> new status -> diverged

        This basically sets the rule to "diverged"
        and blocks expansion process
      """

    # Solvers
    security.declareProtected(Permissions.View, 'isDivergent')
    def isDivergent(self, applied_rule):
      """
        Returns 1 if divergent rule
      """

    security.declareProtected(Permissions.View, 'getDivergenceList')
    def getDivergenceList(self, applied_rule):
      """
        Returns a list Divergence descriptors
      """

    security.declareProtected(Permissions.View, 'getSolverList')
    def getSolverList(self, applied_rule):
      """
        Returns a list Divergence solvers
      """
