##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
""" Portal class

$Id: ERP5Site.py,v 1.5 2003/08/14 08:41:53 smets Exp $
"""

import Globals
#from Products.ERP5 import content_classes
from Products.CMFDefault.Portal import CMFSite, PortalGenerator
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_base, aq_parent, aq_inner, aq_acquire

import threading

from zLOG import LOG

#factory_type_information = []
#for c in content_classes:
#  factory_type_information.append(getattr(c, 'factory_type_information', []))

manage_addERP5SiteForm = Globals.HTMLFile('dtml/addPortal', globals())
manage_addERP5SiteForm.__name__ = 'addPortal'

class ERP5Site ( CMFSite ):
    """
        The *only* function this class should have is to help in the setup
        of a new ERP5.  It should not assist in the functionality at all.
    """
    meta_type = 'ERP5 Site'
    constructors = (manage_addERP5SiteForm, )
    uid = 0
    last_id = 0
    icon = 'portal.gif'

    _properties = (
        {'id':'title', 'type':'string'},
        {'id':'description', 'type':'text'},
        )
    title = ''
    description = ''

    __ac_permissions__=( ( 'Manage portal', ('manage_migrate_content','generateNewId') )
                       , ( 'Add portal content', () )
                       , ( 'Add portal folders', () )
                       , ( 'List portal members', () )
                       , ( 'Reply to item', () )
                       , ( 'View',  ( 'view', 'isEffective', 'getUid', 'getParentUid', 'searchFolder') )
                       )

    def view(self):
        """
          Returns the default view.
          Implemented for consistency
        """
        return self.index_html()

    def getUid(self):
      """
        Returns the UID of the object. Eventually reindexes
        the object in order to make sure there is a UID
        (useful for import / export).

        WARNING : must be updates for circular references issues
      """
      #if not hasattr(self, 'uid'):
      #  self.reindexObject()
      return getattr(self, 'uid', 0)

    def getParentUid(self):
      """
        A portal has no parent
      """
      return self.getUid()

    def searchFolder(self, **kw):
      """
        Search the content of a folder by calling
        the portal_catalog.
      """
      if not kw.has_key('parent_uid'):
        kw['parent_uid'] = self.uid
      kw2 = {}
      # Remove useless matter before calling the
      # catalog. In particular, consider empty
      # strings as None values
      for cname in kw.keys():
        if kw[cname] != '' and kw[cname]!=None:
          kw2[cname] = kw[cname]
      # The method to call to search the folder
      # content has to be called z_search_folder
      method = self.portal_catalog.z_search_folder
      return method(**kw2)

    def generateNewId(self):
        """
          Generate a new Id which has not been taken yet in this folder.
          Eventually increment the id number until an available id
          can be found
        """
        my_id = self.last_id
        l = threading.Lock()
        l.acquire()
        try:
          while hasattr(self,str(my_id)):
            self.last_id = self.last_id + 1
            my_id = self.last_id
        finally:
          l.release()
        return str(my_id)

    # Proxy methods for security reasons
    def getOwnerInfo(self):
      return self.owner_info()

    def getPhysicalPath(self):
      """
      We can have some crasy things with the default
      getPhysicalPath, when for example we tried
      http://localhost:9673/toto/toto/toto/getPhysicalPath we had
      ('', 'toto', 'toto', 'toto') instead of ('', 'toto')
      So we have to rewrite this method
      """
      path = (self.__name__,)

      p = aq_parent(aq_inner(self))

      # JPS: We use uid instead of getUid so that it is compatible with CMFSite
      while p is not None and hasattr(p,'uid') and \
            getattr(p, 'uid', None) == getattr(self, 'uid', None):
        p = aq_parent(aq_inner(p))
      if p is not None:
        path = p.getPhysicalPath() + path

      return path



    # Make sure fixConsistency is recursive - ERROR - this creates recursion errors
    # checkConsistency = Folder.checkConsistency
    # fixConsistency = Folder.fixConsistency

Globals.InitializeClass(ERP5Site)

class ERP5Generator(PortalGenerator):

    klass = ERP5Site

    def setupTools(self, p):
        """Set up initial tools"""

        PortalGenerator.setupTools(self, p)

        addCMFDefaultTool = p.manage_addProduct['ERP5'].manage_addTool
        addCMFDefaultTool('ERP5 Categories', None)
        addCMFDefaultTool('ERP5 Selections', None)
        addCMFDefaultTool('ERP5 Synchronizations', None)

    #def setupMembersFolder(self, p):
    #    PortalFolder.manage_addPortalFolder(p, 'Members')
    #    p.Members.manage_addProduct['OFSP'].manage_addDTMLMethod(
    #        'index_html', 'Member list', '<dtml-return roster>')


    #def setupDefaultSkins(self, p):
    #    PortalGenerator.setupDefaultSkins(self, p)
    #    ps = getToolByName(p, 'portal_skins')
        #ps.addSkinSelection('Nouvelle',
        #    'nouvelle, custom, topic, content, generic, control, Images')

    #def setupTypes(self, p, initial_types=factory_type_information):
    #    method = PortalGenerator.setupTypes
    #    #method(self, p, factory_type_information)
    #    LOG("Type", 0, str(factory_type_information))
    #    PortalGenerator.setupTypes(self, p, {'initial_types':factory_type_information})

def manage_addERP5Site(self, id, title='ERP5', description='',
                         create_userfolder=1,
                         email_from_address='postmaster@localhost',
                         email_from_name='Portal Administrator',
                         validate_email=0, RESPONSE=None):
    '''
    Adds a portal instance.
    '''
    gen = ERP5Generator()
    from string import strip
    id = strip(id)
    p = gen.create(self, id, create_userfolder)
    gen.setupDefaultProperties(p, title, description,
                               email_from_address, email_from_name,
                               validate_email)
    if RESPONSE is not None:
        RESPONSE.redirect(p.absolute_url() + '/finish_portal_construction')

# Patch the standard method
CMFSite.getPhysicalPath = ERP5Site.getPhysicalPath